#include "collection.h"

using namespace std;

namespace Msp {
namespace DataFile {

Collection::~Collection()
{
	for(TypeList::iterator i = types.begin(); i!=types.end(); ++i)
		delete *i;
}

void Collection::add_future(const std::string &name)
{
	if(items.count(name))
		throw key_error(typeid(ItemMap));

	for(TypeList::const_iterator i=types.begin(); i!=types.end(); ++i)
		if((*i)->match_name(name))
		{
			items.insert(ItemMap::value_type(name, (*i)->create_future()));
			return;
		}

	/* XXX throw something?  If we do, DirectoryCollection needs some way to
	check if a name matches any item type. */
}


Collection::Loader::Loader(Collection &c):
	coll(c)
{	
	for(TypeList::const_iterator i = coll.types.begin(); i!=coll.types.end(); ++i)
		(*i)->add_to_loader(*this);
}


CollectionItemTypeBase::CollectionItemTypeBase():
	tag(0)
{ }

CollectionItemTypeBase::~CollectionItemTypeBase()
{
	delete tag;
}

void CollectionItemTypeBase::set_keyword(const string &k)
{
	kwd = k;
	if(suffixes.empty())
		add_suffix("."+kwd);
}

void CollectionItemTypeBase::add_suffix(const string &s)
{
	suffixes.push_back(s);
}

bool CollectionItemTypeBase::match_name(const string &name) const
{
	for(vector<string>::const_iterator i=suffixes.begin(); i!=suffixes.end(); ++i)
		if(name.size()>i->size() && !name.compare(name.size()-i->size(), string::npos, *i))
			return true;
	return false;
}

} // namespace DataFile
} // namespace Msp
